/*	stl.cpp

	Copyright (C) 2000-2003 Robert A. van Engelen. All Rights Reserved.

	To use this package, import "stl.h". For example:

	#import "stl.h"

	Run the gSOAP compiler on your header file with option -i:

	soapcpp2 -i file.h

	Compile and link stl.cpp with your application.  You will have to
	recompile and link the stl.cpp source for every application that
	you build with it, because your header file determines the STL string
	types that you use (xsd__string and xsd__string_).
*/

#include "soapH.h"

////////////////////////////////////////////////////////////////////////////////
//
//	std::string
//
////////////////////////////////////////////////////////////////////////////////

#ifdef SOAP_TYPE_xsd__string

SOAP_FMAC1 void SOAP_FMAC2 soap_mark_xsd__string(struct soap *soap, const std::string *s)
{ }

SOAP_FMAC1 void SOAP_FMAC2 soap_default_xsd__string(struct soap *soap, std::string *s)
{ s->erase();
}

SOAP_FMAC1 int SOAP_FMAC2 soap_out_xsd__string(struct soap *soap, const char *tag, int id, const std::string *s, const char *type)
{ if (soap_element_begin_out(soap, tag, soap_embedded_id(soap, id, s, SOAP_TYPE_xsd__string), type)
   || soap_string_out(soap, s->c_str(), 0) // 0: output as XML element cdata 
   || soap_element_end_out(soap, tag))
    return soap->error;
  return SOAP_OK;
}

SOAP_FMAC1 std::string * SOAP_FMAC2 soap_in_xsd__string(struct soap *soap, const char *tag, std::string *s, const char *type)
{ if (soap_element_begin_in(soap, tag))	// match inbound tag, return if mismatch
    return NULL;
  if (!s)	// allocate object
    s = soap_new_xsd__string(soap, -1);
  if (soap->null)	// inbound xsi:nil element:
    if (s)
      s->erase();
  if (*soap->type && soap_match_tag(soap, soap->type, type))	// check inbound xsi:type
  { soap->error = SOAP_TYPE_MISMATCH;
    soap_revert(soap);	// reject this already accepted XML element, so others can try to match it
    return NULL;
  }
  if (soap->body && !*soap->href)	// XML element has a body (and no ref)
  { s = (std::string*)soap_class_id_enter(soap, soap->id, s, SOAP_TYPE_xsd__string, sizeof(std::string), soap->type, soap->arrayType);
    if (s)
      s->assign(soap_string_in(soap, 1));	// read string, convert (param 0=take literal XML 1=convert), and assign
  }
  else
    s = (std::string*)soap_id_forward(soap, soap->href, soap_class_id_enter(soap, soap->id, s, SOAP_TYPE_xsd__string, sizeof(std::string), soap->type, soap->arrayType), SOAP_TYPE_xsd__string, sizeof(std::string));	// if a ref is present, forward location for backpatching
  if (soap->body && soap_element_end_in(soap, tag))	// match inbound tag end, if present
    return NULL;
  return s;
}

#endif

////////////////////////////////////////////////////////////////////////////////
//
//	std::wstring
//
////////////////////////////////////////////////////////////////////////////////

#ifdef SOAP_TYPE_xsd__string_

SOAP_FMAC1 void SOAP_FMAC2 soap_mark_xsd__string_(struct soap *soap, const std::wstring *s)
{ }

SOAP_FMAC1 void SOAP_FMAC2 soap_default_xsd__string_(struct soap *soap, std::wstring *s)
{ s->erase();
}

SOAP_FMAC1 int SOAP_FMAC2 soap_out_xsd__string_(struct soap *soap, const char *tag, int id, const std::wstring *s, const char *type)
{ if (soap_element_begin_out(soap, tag, soap_embedded_id(soap, id, s, SOAP_TYPE_xsd__string_), type)
   || soap_wstring_out(soap, s->c_str(), 0)
   || soap_element_end_out(soap, tag))
    return soap->error;
  return SOAP_OK;
}

SOAP_FMAC1 std::wstring * SOAP_FMAC2 soap_in_xsd__string_(struct soap *soap, const char *tag, std::wstring *s, const char *type)
{ if (soap_element_begin_in(soap, tag))	// match inbound tag, return if mismatch
    return NULL;
  if (!s)	// allocate object
    s = soap_new_xsd__string_(soap, -1);
  if (soap->null)	// inbound xsi:nil element:
    if (s)
      s->erase();
  if (*soap->type && soap_match_tag(soap, soap->type, type))	// check inbound xsi:type
  { soap->error = SOAP_TYPE_MISMATCH;
    soap_revert(soap);	// reject this already accepted XML element, so others can try to match it
    return NULL;
  }
  if (soap->body && !*soap->href)	// XML element has a body (and no ref)
  { s = (std::wstring*)soap_class_id_enter(soap, soap->id, s, SOAP_TYPE_xsd__string_, sizeof(std::wstring), soap->type, soap->arrayType);
    if (s)
      s->assign(soap_wstring_in(soap, 1));	// read string, convert (param 0=take literal XML 1=convert), and assign
  }
  else
    s = (std::wstring*)soap_id_forward(soap, soap->href, soap_class_id_enter(soap, soap->id, s, SOAP_TYPE_xsd__string_, sizeof(std::wstring), soap->type, soap->arrayType), SOAP_TYPE_xsd__string_, sizeof(std::wstring));	// if a ref is present, forward location for backpatching
  if (soap->body && soap_element_end_in(soap, tag))	// match inbound tag end, if present
    return NULL;
  return s;
}

#endif
